------------------------------------------------------------------------
-- A thingy thing to make Animations and AnimationGroups a bit easier
-- to create.
-- Barely sufficient for the purposes of NamePlateGrid.
-- 
-- Nothing gets cached or reused. NPG will simply create a new
-- animation and orphan the old one.
------------------------------------------------------------------------

local ADDON_NAME, NPG = ...

local tonumber = _G.tonumber
local wipe = _G.wipe

NPG.Animation = {}
do
   local ANIM_FUNCS = {}
   
   ANIM_FUNCS.alpha = function(data, alpha)
      local a = data.ag:CreateAnimation("Alpha")
      alpha = tonumber(alpha) or 0
      a:SetChange(alpha)
      return a
   end
   
   ANIM_FUNCS.a = ANIM_FUNCS.alpha
   
   ANIM_FUNCS.translation = function(data, ofsX, ofsY)
      local a = data.ag:CreateAnimation("Translation")
      ofsX = tonumber(ofsX) or 0
      ofsY = tonumber(ofsY) or 0
      a:SetOffset(ofsX, ofsY)
      return a
   end
   
   ANIM_FUNCS.t = ANIM_FUNCS.translation
   
   ANIM_FUNCS.duration = function(data, duration)
      data.duration = tonumber(duration) or 0
   end
   
   ANIM_FUNCS.d = ANIM_FUNCS.duration
   
   ANIM_FUNCS.smoothing = function(data, smoothing)
      data.smoothing = smoothing
   end
   
   ANIM_FUNCS.sm = ANIM_FUNCS.smoothing
   
   ANIM_FUNCS.scale = function(data, scaleX, scaleY)
      local a = data.ag:CreateAnimation("Scale")
      scaleX = tonumber(scaleX) or 0
      scaleY = tonumber(scaleY) or 0
      a:SetScale(scaleX, scaleY)
   end
   
   ANIM_FUNCS.s = ANIM_FUNCS.scale
   
   local AnimData = {}
   
   function NPG.Animation:CreateAnimation(frame, animString)
      wipe(AnimData)
      
      AnimData.order = 0
      AnimData.smoothing = "IN"
      AnimData.ag = frame:CreateAnimationGroup()
      AnimData.ag:SetLooping("NONE")
      AnimData.ag:SetIgnoreFramerateThrottle(true)
      
      for anim in animString:gmatch("[^%+]+") do
         AnimData.duration = 0
         AnimData.order = AnimData.order + 1
         for func, params in anim:gmatch(" *(%w+) *%(([^%)]*)%)") do
            local a = ANIM_FUNCS[func](AnimData, strsplit(",", params))
            if a then
               a:SetDuration(AnimData.duration)
               a:SetOrder(AnimData.order)
               a:SetSmoothing(AnimData.smoothing)
            end
         end
      end
      
      return AnimData.ag
   end
end
